@@ -1,5 +1,6 @@
 #include "icon.h"
 
+#include <ctype.h>
 #include <stdlib.h>
 #include <stdio.h>
 #include <string.h>
@@ -44,7 +45,7 @@ dir_context_is_allowed(const char *context)
     static const char *const allowed_contexts[] = {"applications", "apps", "legacy"};
 
     if (context == NULL)
-        return NULL;
+        return false;
 
     for (size_t i = 0; i < sizeof(allowed_contexts) / sizeof(allowed_contexts[0]); i++) {
         if (strcasecmp(context, allowed_contexts[i]) == 0)
@@ -126,10 +127,49 @@ parse_theme(FILE *index, struct icon_theme *theme, bool filter_context,
             continue;
         }
 
-        char *tok_ctx = NULL;
+        char *p = line;
+        while (isspace((unsigned char)*p))
+            p++;
+        /* skip over comments and blank lines comprised of only whitespace */
+        if (*p == '#' || *p == '\0') {
+            free(line);
+            continue;
+        }
 
-        const char *key = strtok_r(line, "=", &tok_ctx);
-        char *value = strtok_r(NULL, "=", &tok_ctx);
+        const char *key_start = p;
+        /* look for the first space or '=', i.e. the end of the key name */
+        while (*p != '\0' && !isspace((unsigned char)*p) && *p != '=')
+            p++;
+        if (p == key_start) {
+            /* empty key name */
+            free(line);
+            continue;
+        }
+
+        char *key_end = p;
+
+        while (isspace((unsigned char)*p))
+            p++;
+
+        /* keys may not contain whitespace */
+        if (*p != '=') {
+            free(line);
+            continue;
+        }
+        /* consume the '=' */
+        p++;
+
+        /*
+         * While values themselves may contain whitespace, the spec tells us to
+         * ignore any leading whitespace:
+         * "Space before and after the equals sign should be ignored"
+         */
+        while (isspace((unsigned char)*p))
+            p++;
+
+        *key_end = '\0';
+        const char *key = key_start;
+        char *value = p;
 
         if (strcasecmp(key, "inherits") == 0) {
             char *ctx = NULL;
